#include <NewPing.h>

#define TRIGGER_PIN      12
#define ECHO_PIN         11
#define MAX_DISTANCE     200
#define ENA              3
#define ENB              5
#define IN1              4
#define IN2              7
#define IN3              8
#define IN4              9
#define SWITCH_PIN       2  // Main switch
#define PIR_PIN          10 // PIR Sensor
#define PUMP_PIN         6  // Water Pump
#define PERSON_PRESENT_TIMEOUT 5000 // Hypothetical time of human existence (ms)

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);

unsigned long lastPersonDetectedTime = 0; // Record the last time the person was detected

void setup() {
  pinMode(ENA, OUTPUT);
  pinMode(ENB, OUTPUT);
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  pinMode(SWITCH_PIN, INPUT_PULLUP);
  pinMode(PIR_PIN, INPUT);
  pinMode(PUMP_PIN, OUTPUT); 
  Serial.begin(9600);
}

void loop() {
  unsigned long currentTime = millis(); // Get current time
  if (digitalRead(SWITCH_PIN) == LOW) { // If the main switch is on
    if (digitalRead(PIR_PIN) == HIGH) { // If a person is detected
      lastPersonDetectedTime = currentTime; // Updated the time when person is detected
    }
    
    if (currentTime - lastPersonDetectedTime <= PERSON_PRESENT_TIMEOUT) { // Assume that the person is still ahead during the timeout period
      unsigned int distance = sonar.ping_cm();
      if (distance > 0 && distance < 30) { // If there's an obstacle 30 centimeters ahead
        stopMotors(); // Stop motor
        digitalWrite(PUMP_PIN, HIGH); // Activate pump to spray water
        delay(1000); // Spray duration, adjust as needed
        digitalWrite(PUMP_PIN, LOW); // Stop spraying
      } else {
        moveForward(); // move forward
        digitalWrite(PUMP_PIN, LOW); // Make sure the pump is off
      }
    } else {
      stopMotors(); // If the timeout period is exceeded, stop the motor
      digitalWrite(PUMP_PIN, LOW); // Make sure the pump is off
    }
  } else {
    stopMotors(); // If the main switch is off, stop the motor
    digitalWrite(PUMP_PIN, LOW); // Make sure the pump is off
  }
}

void stopMotors() {
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, LOW);
  analogWrite(ENA, 0); 
  analogWrite(ENB, 0);
}

void moveForward() {
  digitalWrite(IN1, HIGH);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, HIGH);
  digitalWrite(IN4, LOW);
  analogWrite(ENA, 255); // Left motor full speed
  analogWrite(ENB, 255); // Right motor full speed
}